/******************************************************************
*  Super amazing PS2 controller Arduino Library v1.4
*		details and example sketch: 
*			http://www.billporter.info/?p=240
*
*    Original code by Shutter on Arduino Forums
*
*    Revamped, made into lib by and supporting continued development:
*              Bill Porter
*              www.billporter.info
*     Modified by KurtE to run as part of my Basic Atom Pro C library work...
*
*	 Contributers:
*		Eric Wetzel (thewetzel@gmail.com)
*
*
*This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
<http://www.gnu.org/licenses/>
*  
*
******************************************************************/

#include "PS2X_lib.h"
#include "wiring.h"

#define delayMicroseconds(uMS)	pause(uMS)


static byte enter_config[]={0x01,0x43,0x00,0x01,0x00};
static byte set_mode[]={0x01,0x44,0x00,0x01,0x03,0x00,0x00,0x00,0x00};
static byte set_bytes_large[]={0x01,0x4F,0x00,0xFF,0xFF,0x03,0x00,0x00,0x00};
static byte exit_config[]={0x01,0x43,0x00,0x00,0x5A,0x5A,0x5A,0x5A,0x5A};
static byte enable_rumble[]={0x01,0x4D,0x00,0x00,0x01};

boolean PS2X::NewButtonState() {
	return ((last_buttons ^ buttons) > 0);

}

boolean PS2X::NewButtonState(unsigned int button) {
	return (((last_buttons ^ buttons) & button) > 0);
}

boolean PS2X::ButtonPressed(unsigned int button) {
	return(NewButtonState(button) & Button(button));
}

boolean PS2X::ButtonReleased(unsigned int button) {
	return((NewButtonState(button)) & ((~last_buttons & button) > 0));
}
  
boolean PS2X::Button(word button) {
	return ((~buttons & button) > 0);
}

unsigned int PS2X::ButtonDataByte() {
	return (~buttons);
}

byte PS2X::Analog(byte button) {
	return PS2data[button];
}
#if 1
unsigned char PS2X::_gamepad_shiftinout (char command) {
        short int i ;
		short int j;
        // pause(1);
        register short int data = 0x00;                                   // clear data variable to save setting low bits later.
        for(i=0;i<8;i++)
        {
			if (command & 0x1)
				PI_HIGH(_piCMD);		//sbi(PORTK, PScommand);      // bit bang "command" out on PScommand wire.
            else 
				PI_LOW(_piCMD);		//cbi(PORTK, PScommand);
            PI_LOW(_piCLK);			//cbi(PORTK, PSclock);                             // CLOCK LOW
			for (j=5; j > 0; j--)
				;
            if ( PI_ISHIGH(_piDAT))		//if((PINK & _BV(PSdata))) 
				data |= (1<<i);		// sbi(data, i); 
            PI_HIGH(_piCLK);			//sbi(PORTK, PSclock);                             // CLOCK HIGH
			command = command >> 1;
        }
        PI_HIGH(_piCMD);				//sbi(PORTK, PScommand);

//        delay_us(20);                                            // wait for ACK to pass.
		for (j=20; j > 0; j--)
				;
		

        return(data);

#else
unsigned char PS2X::_gamepad_shiftinout (char byte) {
	unsigned char tmp = 0;
	for(i=0;i<8;i++) {
		if(CHK(byte,i)) PI_HIGH(_piCMD);
		else  PI_LOW(_piCMD);
		PI_LOW(_piCLK);
		delayMicroseconds(CTRL_CLK);
        if ( PI_ISHIGH(_piDAT))		//if((PINK & _BV(PSdata))) 
			tmp |= (1<<i);		// sbi(data, i); 
		PI_HIGH(_piCLK);
	}
	PI_HIGH(_piCMD);
	delayMicroseconds(CTRL_BYTE_DELAY);
	return tmp;
#endif
}

void PS2X::read_gamepad() {
	read_gamepad(false, 0x00);
}


void PS2X::read_gamepad(boolean motor1, byte motor2) {
	unsigned long temp = millis() - last_read;
  
	if (temp > 1500) //waited to long
		reconfig_gamepad();
    
	if(temp < read_delay)  //waited too short
		delay(read_delay - temp);
    
    
	last_buttons = buttons; //store the previous buttons states

	if(motor2 != 0x00)
		motor2 = map(motor2,0,255,0x40,0xFF); //noting below 40 will make it spin
    
	PI_HIGH(_piCMD);
	PI_HIGH(_piCLK);
	PI_LOW(_piSEL); // low enable joystick
	delayMicroseconds(CTRL_BYTE_DELAY);
	//Send the command to send button and joystick data;
	char dword[9] = {0x01,0x42,0,motor1,motor2,0,0,0,0};
	byte dword2[12] = {0,0,0,0,0,0,0,0,0,0,0,0};
#ifdef PS2X_COM_DEBUG
	DBGPrintf("OUT:IN\n\r");
#endif
	for (int i = 0; i<9; i++) {
		PS2data[i] = _gamepad_shiftinout(dword[i]);
#ifdef PS2X_COM_DEBUG
		DBGPrintf("%x:%x ", dword[i], PS2data[i]);
#endif
	}
	if(PS2data[1] == 0x79) {  //if controller is in full data return mode, get the rest of data
	   	for (int i = 0; i<12; i++) {
			PS2data[i+9] = _gamepad_shiftinout(dword2[i]);
#ifdef PS2X_COM_DEBUG
			DBGPrintf("%x:%x ", dword[i], PS2data[i]);
#endif
	   	}
	}
#ifdef PS2X_COM_DEBUG
	DBGPrintf("\n\r");
#endif
	
	PI_HIGH(_piSEL); // HI disable joystick
	buttons = (PS2data[4] << 8) | PS2data[3] ;
	//buttons = *(word*)(PS2data+3);   //store as one value for multiple functions
	last_read = millis();
}

byte PS2X::config_gamepad(byte clk, byte cmd, byte att, byte dat) {

  	// First pass see if using our array of Porttable items with constants will generate same code as local copy...
	_piCMD = (*g_pPortTable)[cmd];		
	_piDAT = (*g_pPortTable)[dat];
	_piSEL = (*g_pPortTable)[att];
	_piCLK = (*g_pPortTable)[clk];

   	input(dat, true);  			// make the dat line input But also enable pull-up
	output(cmd);				// Make command pin be output
	output(att);				// init the select pin
	output(clk);				// Make clock be output

//  	pinMode(clk, OUTPUT); //configure ports
//  	pinMode(att, OUTPUT);
//  	pinMode(cmd, OUTPUT);
//  	pinMode(dat, INPUT);

	PI_HIGH(_piCMD); 
   	PI_HIGH(_piCLK);
   
   //new error checking. First, read gamepad a few times to see if it's talking
   read_gamepad();
   read_gamepad();
   
   //see if it talked
   if(PS2data[1] != 0x41 && PS2data[1] != 0x73 && PS2data[1] != 0x79){ //see if mode came back. If still anything but 41, 73 or 79, then it's not talking
      #ifdef PS2X_DEBUG
		DBGPrintf("Controller mode not matched or no controller found\n\r");
		DBGPrintf("Expected 0x41 or 0x73, got %x\n\r", PS2data[1]);
	  #endif
	 
	 return 1; //return error code 1
	}
  
  //try setting mode, increasing delays if need be. 
  
  for(int y = 0; y <= 10; y++)
  {
   sendCommandString(enter_config, sizeof(enter_config));
   sendCommandString(set_mode, sizeof(set_mode));
   sendCommandString(exit_config, sizeof(exit_config));
   
   read_gamepad();
   
    if(PS2data[1] == 0x73)
      break;
      
    if(y == 10){
		#ifdef PS2X_DEBUG
		DBGPrintf("Controller accepting commands\n\r");
		DBGPrintf("mode stil set at %x\n\r", PS2data[i]);
		#endif
      return 2; //exit function with error
	  }
    
    read_delay += 10; //add 10ms to read_delay
  }
   
 return 0; //no error if here
}

void PS2X::sendCommandString(byte string[], byte len) {
  
   PI_LOW(_piSEL); // low enable joystick
  for (int y=0; y < len; y++)
    _gamepad_shiftinout(string[y]);
    
   PI_HIGH(_piSEL); //high disable joystick  
   delay(read_delay);                  //wait a few
}

 
byte PS2X::maskToBitNum(byte mask) {
    for (int y = 0; y < 8; y++)
    {
      if(CHK(mask,y))
        return y;
    }
    return 0;
}

void PS2X::enableRumble() {
  
     sendCommandString(enter_config, sizeof(enter_config));
     sendCommandString(enable_rumble, sizeof(enable_rumble));
     sendCommandString(exit_config, sizeof(exit_config));
     en_Rumble = true;
  
}

void PS2X::enablePressures() {
  
     sendCommandString(enter_config, sizeof(enter_config));
     sendCommandString(set_bytes_large, sizeof(set_bytes_large));
     sendCommandString(exit_config, sizeof(exit_config));
     en_Pressures = true;
}

void PS2X::reconfig_gamepad(){
  
   sendCommandString(enter_config, sizeof(enter_config));
   sendCommandString(set_mode, sizeof(set_mode));
   if (en_Rumble)
      sendCommandString(enable_rumble, sizeof(enable_rumble));
   if (en_Pressures)
      sendCommandString(set_bytes_large, sizeof(set_bytes_large));
   sendCommandString(exit_config, sizeof(exit_config));
   
}
