//==============================================================================
// HSIO - Hardware serial port(s) IO. 
// 		some of definitions/functions from Minionbuilder
// 		work to integrate it with my own stuff and including interrupt based
//		serial port support.
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 		Robert V Mahoney Jr
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================
#ifndef _HSIO_H_
#define _HSIO_H_
#include "iopins.h"

#ifdef __cplusplus
extern "C" {
#endif

//-----------------------------------------------------------------------------
// Type definitions
//-----------------------------------------------------------------------------
// Define SetHSerial options:
#define H8DATABITS		0	// 8 data bits  - default
#define H7DATABITS		1	// 7 data bits
#define HNOPARITY		0	// No parity - Default
#define HEVENPARITY		2	// odd parity
#define HODDPARITY		3	// Even parity...
#define H1STOPBITS		0	// 1 stop bit is sent - default
#define H2STOPBITS		1	// 2 stop bits is sent

typedef enum {
	HSERSTAT_CLEAR_INPUT_BUFFER,
	HSERSTAT_CLEAR_OUTPUT_BUFFER,
	HSERSTAT_CLEAR_BOTH_BUFFERS,
	HSERSTAT_INPUT_AVAILABLE,
	HSERSTAT_OUTPUT_EMPTY
} HSERSTAT_OPTIONS;	

// The new format of HSerial commands now have one set of commands the service all of the hardware serial
// ports with which logical port passed in, versus duplicating the commands for each serial port...
extern bool enablehserial (u8 bPortnum=1);
extern bool sethserial (u8 bPortnum, u32 _lbaudrate, u8 _bdatabits=H8DATABITS, u8 _bparity=HNOPARITY, u8 _bstopbits=H1STOPBITS);
extern bool hserout (u8 bPortnum, u8 *_psdata, u16 _wcount);
extern u16 hserin (u8 bPortnum, u32 _ltimeout, u8 *_psdata, u16 _rcount, u16 _bterminator=0xffff);
extern bool hserstat(u8 bPortnum, HSERSTAT_OPTIONS ho);  // ? - would it be better to simply have a function for each???
extern byte hseravail(u8 bPortnum=1);	// How many characters are available?

//extern void hsio_setbaud (u32 _lbaudrate);

#ifdef __cplusplus
}
#endif


#endif	/* _HSIO_H_ */
