//==============================================================================
// Servo.cpp - This file contains servo support that hopefully should be similiar
//		to the hservo system used in basic.  Will first start off just getting
//		servos to work, will then add in the timed group moves.
//  Not sure if unit base.  I am thinking in uSs as that is what most people
//	are used to...  Internally we will convert everything to clock tick units...
//
// Version 0.1 - Work in Progress
//
// Major Contributers: Kurt Eckhardt 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//==============================================================================

// >>>>>>>>>>>>>>>>>>>>>>> Warning needs work!!!!! <<<<<<<<<<<<<<<<<<<<<<<<<<<<

#include "baplib.h"
#include "Servo.h"
//#define DEBUG_HSERVO_COMMANDS
//#define DEBUG_SERVO_PULSE
#ifdef DEBUG_SERVO_PULSE
#define DSERVOTOGGLE(_pn_) toggle(_pn_)
#else
#define DSERVOTOGGLE(_pn_) 
#endif


// For each servo we will store
// 
#ifdef BASICATOMPRO28
#define NUMSERVOS				20	// We are set up for a maximum of 20 servos
#define NUMTIMERCOMPAREMATCH	4	// Number of timer compare match units we have
#else
#define NUMSERVOS				32	// We are set up for a maximum of 20 servos
#define NUMTIMERCOMPAREMATCH	8	// Number of timer compare match units we have
#endif

#define NUMOVFPERCYCLE			5	// number of cycles of the timer per 20ms servo cycle
typedef struct _hsitem
{
	u16		wPW;			// Current pulse width in clocks
	u16		wPWEnd;			// Desired pulse width after a group move
	u16		wDeltaPW;		// how much to change per cycle
} HSITEM;

typedef struct _hsevero_data
{
	HSITEM	ahsi[NUMSERVOS];
	u16		wGroupMovePW[NUMSERVOS];	// place to remember new values for a group move.
	u8		cServosActive;		// Are there any servos active?
	u8 		bGroupMove;			// are we in a group move? - keep a counter so can nest...
	u8		iServo;				// which servo to start with for this pass
	u8		wFudge;				// fudge factor to get closer to exact pulses
	u8		bWhichIntsToProcess; // combination of which are enabled and which have happened.
	u8		abActivePins[NUMTIMERCOMPAREMATCH];	// know which pins are active. = 0xff if not active...

	u8		bChannel;			// which channel of servos are we processing.
	u32		ulTimerOverflow;	// Count of timer overflows.
} HSERVO_DATA;

HSERVO_DATA g_hservoD;

#ifdef H8_FAMILY_3687
extern "C" {
typedef void (*PINTHANDLER)(void);
}
extern PINTHANDLER UpdateHandler(int iVect, PINTHANDLER pinth);
extern "C" void	TIMERZ0_ARC32(void);

const u8 g_aabMuXServoMapTable[4][8] = {
		{2,    7,      10,     16,    18,      20,    28,     25},
		{1,    4,       9,     12,    17,      23,     31,     26},
		{0,    6,       8,     14,     16,      21,     29,     27},
		{3,    5,       11,    13,     19,      22,     30,     24}
};		
#endif

#define clockCyclesPerMicrosecond (F_CPU/1000000L)

//#define ticksToUs(_ticks) (( (unsigned long)_ticks * 8)/ clockCyclesPerMicrosecond ) // converts from ticks back to microseconds
#define usToTicks(_us)    (unsigned int)(((unsigned long) clockCyclesPerMicrosecond * _us) / 8)     // converts microseconds to tick (assumes prescale of 8)  
#define ticksToUs(_ticks) (( (unsigned long)_ticks * 8)/ (unsigned long)clockCyclesPerMicrosecond ) // converts from ticks back to microseconds



#define TRIM_DURATION       2                               // compensation ticks to trim adjust for digitalWrite delays // 12 August 2009


//=============================================================================
// Inline function that is called to update each servos position in handling
// a timed servo move
//=============================================================================
__inline__ void IncrementalUpdateServoPos(HSITEM* phsi) __attribute__ ((always_inline));
__inline__ void IncrementalUpdateServoPos(HSITEM* phsi)
{
	// Also see if we need to adjust the pulse width for the next time...
	if (phsi->wDeltaPW) {
		if (phsi->wPW < phsi->wPWEnd) {
			phsi->wPW += phsi->wDeltaPW;
			if (phsi->wPW >= phsi->wPWEnd){ // reached the end point
				phsi->wPW = phsi->wPWEnd;
				phsi->wDeltaPW = 0;
			}
		} else {
			phsi->wPW -= phsi->wDeltaPW;
			if (phsi->wPW <= phsi->wPWEnd){ // reached the end point
				phsi->wPW = phsi->wPWEnd;
				phsi->wDeltaPW = 0;
			}
		}
	}
}

#ifdef BASICATOMPRO28
//=============================================================================
// TIMERW() - Main timerw functino that is used to handle the background processing
// of the servo sub-system
//=============================================================================
extern "C" void TIMERW() __attribute__ ((interrupt_handler));
extern "C" void TIMERW()
{
	DBGPrintf("***TIMERW***\r");
	// See why we woke up for
	if (TW.TSRW.BIT.OVF) {
		// See if we need to set any of the pins back low now.
		if (g_hservoD.abActivePins[0] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[0]]);
		if (g_hservoD.abActivePins[1] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[1]]);
		if (g_hservoD.abActivePins[2] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[2]]);
		if (g_hservoD.abActivePins[3] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[3]]);

		// If no servos are currently active we can now bail as we have
		// set all of the previously active servos back to low...
		if (!g_hservoD.cServosActive) {
			TW.TSRW.BYTE = 0;		// turn off all of the status bytes.
			TW.TIERW.BYTE = 0;		// Turn off all interrupts
			return;					// and return...
		}

		// Clock over run.  we are ready to setup for the next pass.
		// setup TCNT such that we interrupt every 4mS.  For each 4ms we handle 1 servo per GRx value so
		// 4 servos.  So every 5 times through is one complete cycle which equals 20ms
		TW.TCNT = 1536-90;				// set such that 65536-1536=64000tics per interrupt/16MperHz = 4000uS= 4ms (fudge)
		g_hservoD.wFudge = 103;				// init the fudge factor...
		g_hservoD.iServo++; 
		if (g_hservoD.iServo >= NUMSERVOS) {
			g_hservoD.iServo = 0;
		}
			
		// Now lets start up to see if which servo we need to process in this pass
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
			
			// servo is active so lets get all of the timers and the like setup
			TW.GRA = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge -= 29;
			TW.TIERW.BIT.IMIEA = 1;		// enable that interrupt
			g_hservoD.abActivePins[0] = g_hservoD.iServo;	// remember which pin we are using
				
		} else 
			g_hservoD.abActivePins[0] = 0xff;	// remember which pin we are using

		// Channel B
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TW.GRB = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge -= 29;
			TW.TIERW.BIT.IMIEB = 1;		// enable that interrupt
			g_hservoD.abActivePins[1] = g_hservoD.iServo;	// remember which pin we are using
		} else 
			g_hservoD.abActivePins[1] = 0xff;	// remember which pin we are using

		// Channel C
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TW.GRC = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge += 29;
			TW.TIERW.BIT.IMIEC = 1;		// enable that interrupt
			g_hservoD.abActivePins[2] = g_hservoD.iServo;	// remember which pin we are using
		} else 
			g_hservoD.abActivePins[2] = 0xff;

		// Channel D
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TW.GRD = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			TW.TIERW.BIT.IMIED = 1;		// enable that interrupt
			g_hservoD.abActivePins[3] = g_hservoD.iServo;	// remember which pin we are using
		}
		else 
			g_hservoD.abActivePins[3] = 0xff;

		TW.TSRW.BYTE = 0;								// Clear all of the status bits
		return;											// and finally return
	}

	// Lets see which of the 4 channels we woke up for.  Lets try with pre-generated bitmask.
	g_hservoD.bWhichIntsToProcess = (TW.TSRW.BYTE & TW.TIERW.BYTE);
	if (g_hservoD.bWhichIntsToProcess & 0x01) {				// GRA Match
	//if (TW.TSRW.BIT.IMFA && TW.TIERW.BIT.IMIEA)	{		// GRA Match? and that interrupt is enabled...
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[0]]);	// set the servo pin high!
		TW.TSRW.BIT.IMFA = 0;	
		TW.TIERW.BIT.IMIEA = 0;								// clear the interrupt request as well
	}

	if (g_hservoD.bWhichIntsToProcess & 0x02) {				// GRB Match
	//if (TW.TSRW.BIT.IMFB && TW.TIERW.BIT.IMIEB)	{		// GRB Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[1]]);	// Like A
		TW.TSRW.BIT.IMFB = 0;	
		TW.TIERW.BIT.IMIEB = 0;	
	}
	
	if (g_hservoD.bWhichIntsToProcess & 0x04) {				// GRC Match
	//if (TW.TSRW.BIT.IMFC && TW.TIERW.BIT.IMIEC) {		// GRC Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[2]]);	// Like A
		TW.TSRW.BIT.IMFC = 0;	
		TW.TIERW.BIT.IMIEC = 0;	
	}

	if (g_hservoD.bWhichIntsToProcess & 0x08) {				// GRD Match
	//if (TW.TSRW.BIT.IMFD && TW.TIERW.BIT.IMIED)	{		// GRD Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[3]]);	// Like A
		TW.TSRW.BIT.IMFD = 0;	
		TW.TIERW.BIT.IMIED = 0;	
	}
}
#else

// This is the HSERVO implementation.
//=============================================================================
// TIMERZ0() - Main timerw functino that is used to handle the background processing
// of the servo sub-system
//=============================================================================
extern "C" void TIMERZ0() __attribute__ ((interrupt_handler));
extern "C" void TIMERZ0()
{
	// quick and dirty simply detect at run time and call..
	if (g_bWhichBap == H8_3687_ARC32) { 
		TIMERZ0_ARC32();
		return;
	}
		
//	DBGPrintf("***TIMERW***\r");
	// See why we woke up for
	if (TZ0.TSR.BIT.OVF) {
		// See if we need to set any of the pins back low now.
		if (g_hservoD.abActivePins[0] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[0]]);
		if (g_hservoD.abActivePins[1] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[1]]);
		if (g_hservoD.abActivePins[2] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[2]]);
		if (g_hservoD.abActivePins[3] != 0xff)
			PI_LOW((*g_pPortTable)[g_hservoD.abActivePins[3]]);

		// If no servos are currently active we can now bail as we have
		// set all of the previously active servos back to low...
		if (!g_hservoD.cServosActive) {
			TZ0.TSR.BYTE = 0;		// turn off all of the status bytes.
			TZ0.TIER.BYTE = 0;		// Turn off all interrupts
			return;					// and return...
		}

		// Clock over run.  we are ready to setup for the next pass.
		// setup TCNT such that we interrupt every 4mS.  For each 4ms we handle 1 servo per GRx value so
		// 4 servos.  So every 5 times through is one complete cycle which equals 20ms
		TZ0.TCNT = 1536-90;				// set such that 65536-1536=64000tics per interrupt/16MperHz = 4000uS= 4ms (fudge)
		g_hservoD.wFudge = 103;				// init the fudge factor...
		g_hservoD.iServo++; 
		if (g_hservoD.iServo >= NUMSERVOS) {
			g_hservoD.iServo = 0;
		}
			
		// Now lets start up to see if which servo we need to process in this pass
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
			
			// servo is active so lets get all of the timers and the like setup
			TZ0.GRA = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge -= 29;
			TZ0.TIER.BIT.IMIEA = 1;		// enable that interrupt
			g_hservoD.abActivePins[0] = g_hservoD.iServo;	// remember which pin we are using
				
		} else 
			g_hservoD.abActivePins[0] = 0xff;	// remember which pin we are using

		// Channel B
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TZ0.GRB = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge -= 29;
			TZ0.TIER.BIT.IMIEB = 1;		// enable that interrupt
			g_hservoD.abActivePins[1] = g_hservoD.iServo;	// remember which pin we are using
		} else 
			g_hservoD.abActivePins[1] = 0xff;	// remember which pin we are using

		// Channel C
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TZ0.GRC = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			g_hservoD.wFudge += 29;
			TZ0.TIER.BIT.IMIEC = 1;		// enable that interrupt
			g_hservoD.abActivePins[2] = g_hservoD.iServo;	// remember which pin we are using
		} else 
			g_hservoD.abActivePins[2] = 0xff;

		// Channel D
		g_hservoD.iServo++;
		if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
			// First see if we need to adjust the serevo position
			IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);

			// servo is active so lets get all of the timers and the like setup
			TZ0.GRD = 65535 - g_hservoD.ahsi[g_hservoD.iServo].wPW - g_hservoD.wFudge;  // off by 1 but should work
			TZ0.TIER.BIT.IMIED = 1;		// enable that interrupt
			g_hservoD.abActivePins[3] = g_hservoD.iServo;	// remember which pin we are using
		}
		else 
			g_hservoD.abActivePins[3] = 0xff;

		TZ0.TSR.BYTE = 0;								// Clear all of the status bits
		return;											// and finally return
	}

	// Lets see which of the 4 channels we woke up for.  Lets try with pre-generated bitmask.
	g_hservoD.bWhichIntsToProcess = (TZ0.TSR.BYTE & TZ0.TIER.BYTE);
	if (g_hservoD.bWhichIntsToProcess & 0x01) {				// GRA Match
	//if (TZ0.TSR.BIT.IMFA && TZ0.TIER.BIT.IMIEA)	{		// GRA Match? and that interrupt is enabled...
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[0]]);	// set the servo pin high!
		TZ0.TSR.BIT.IMFA = 0;	
		TZ0.TIER.BIT.IMIEA = 0;								// clear the interrupt request as well
	}

	if (g_hservoD.bWhichIntsToProcess & 0x02) {				// GRB Match
	//if (TZ0.TSR.BIT.IMFB && TZ0.TIER.BIT.IMIEB)	{		// GRB Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[1]]);	// Like A
		TZ0.TSR.BIT.IMFB = 0;	
		TZ0.TIER.BIT.IMIEB = 0;	
	}
	
	if (g_hservoD.bWhichIntsToProcess & 0x04) {				// GRC Match
	//if (TZ0.TSR.BIT.IMFC && TZ0.TIER.BIT.IMIEC) {		// GRC Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[2]]);	// Like A
		TZ0.TSR.BIT.IMFC = 0;	
		TZ0.TIER.BIT.IMIEC = 0;	
	}

	if (g_hservoD.bWhichIntsToProcess & 0x08) {				// GRD Match
	//if (TZ0.TSR.BIT.IMFD && TZ0.TIER.BIT.IMIED)	{		// GRD Match?
		PI_HIGH((*g_pPortTable)[g_hservoD.abActivePins[3]]);	// Like A
		TZ0.TSR.BIT.IMFD = 0;	
		TZ0.TIER.BIT.IMIED = 0;	
	}
}

//=============================================================================
// TIMERZ0-Arc32() - Main timerw function that is used to handle the background processing
// of the servo sub-system
//=============================================================================
#ifdef LATER // try to switch the interrupt vectors.  For now lets simply handle as a call/return
extern "C" void TIMERZ0_ARC32() __attribute__ ((interrupt_handler));
#endif
extern "C" void TIMERZ0_ARC32()
{
	int i;
	// See why we woke up for - Do the easy ones first. If we get a GRA-D match, than we should have already
	// set the output pin high, simply set the next match to something just before the counter overflow
	// and set up the pin to go back low at that point...
	DSERVOTOGGLE(40);		// BUGBUG:: For debug servo stuff.

	if (TZ0.TSR.BIT.IMFA && TZ0.TIER.BIT.IMIEA) {
		DSERVOTOGGLE(41);		// BUGBUG:: For debug servo stuff.

		TZ0.GRA = 0xffff;			// should trigger and do its processing before overflow happens.
		TZ0.TIORA.BIT.IOA = 0x001;	// Will set to zero when timer happens
		TZ0.TIER.BIT.IMIEA = 0;		// don't need interrupt here
		TZ0.TSR.BIT.IMFA = 0;
	}	
	if (TZ0.TSR.BIT.IMFB && TZ0.TIER.BIT.IMIEB) {
		DSERVOTOGGLE(41);		// BUGBUG:: For debug servo stuff.
		TZ0.GRB = 0xffff;			// should trigger and do its processing before overflow happens.
		TZ0.TIORA.BIT.IOB = 0x001;	// Will set to zero when timer happens
		TZ0.TIER.BIT.IMIEB = 0;		// don't need interrupt here
		TZ0.TSR.BIT.IMFB = 0;
	}	
	if (TZ0.TSR.BIT.IMFC && TZ0.TIER.BIT.IMIEC) {
		DSERVOTOGGLE(41);		// BUGBUG:: For debug servo stuff.
		TZ0.GRC = 0xffff;			// should trigger and do its processing before overflow happens.
		TZ0.TIORC.BIT.IOC = 0x001;	// Will set to zero when timer happens
		TZ0.TIER.BIT.IMIEC = 0;		// don't need interrupt here
		TZ0.TSR.BIT.IMFC = 0;
	}	

	if (TZ0.TSR.BIT.IMFD && TZ0.TIER.BIT.IMIED) {
		DSERVOTOGGLE(41);		// BUGBUG:: For debug servo stuff.
		TZ0.GRD = 0xffff;			// should trigger and do its processing before overflow happens.
		TZ0.TIORC.BIT.IOD = 0x001;	// Will set to zero when timer happens
		TZ0.TIER.BIT.IMIED = 0;		// don't need interrupt here
		TZ0.TSR.BIT.IMFD = 0;
	}	
	
	// OK the main work horse.
	if (TZ0.TSR.BIT.OVF) {			
		g_hservoD.ulTimerOverflow++;		// timer overflow counter.
		TZ0.TSR.BIT.OVF = 0;				// Clear the overflow flag			

		// OK, before we switch to a new channel, we may need to drive the signals low that have
		// a backing IO pin as they do not have a pull-down resistor anymore or it may not be
		// strong enough to counter pull-up resistors in some servos...
		for (i=0; i< NUMTIMERCOMPAREMATCH; i++) {			// BUGBUG:: Unwind these functions into actual quick code...
			if (g_hservoD.abActivePins[i] != 0xff) {
				output(g_hservoD.abActivePins[i]);
				low (g_hservoD.abActivePins[i]);
				g_hservoD.abActivePins[i] = 0xff; // only process once...
			}
		}

		g_hservoD.bChannel++;				// increment the channel.
		if (g_hservoD.bChannel > 5) // handle wrap
			g_hservoD.bChannel = 0;
		if (g_hservoD.bChannel <= 3) {
			// Setup for the next channel.
			// Note sure of order...
			// BUGBUG:: speed this up by breaking it down to actual bit set...
			setpin(47, (g_hservoD.bChannel >> 1));	// set it to high order bit
			setpin(46, (g_hservoD.bChannel & 1));		// set to low order bit
			TZ0.TIORA.BYTE = 0x0;		// Tell all channels we are not doing any output
			TZ0.TIORC.BYTE = 0x0;
			TZ1.TIORA.BYTE = 0x0;
			TZ1.TIORC.BYTE = 0x0;
			
			// Now lets process the 4/8 different IO pins for this channel.
			// Start off with just 4 pins and then do all code inline, next
			// see if better table drive...


#define A32_TZF	60
			g_hservoD.iServo = g_aabMuXServoMapTable[g_hservoD.bChannel][0];
			if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
				IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
				TZ0.GRA = (65535 - A32_TZF) - g_hservoD.ahsi[g_hservoD.iServo].wPW;  // 
				TZ0.TSR.BIT.IMFA = 0;		// Make sure the flag is clear
				TZ0.TIER.BIT.IMIEA = 1;		// enable that interrupt
				//TZ.TOCR.BIT.TOA0 = 0;		// Probably can just set this at init.
				TZ.TOER.BIT.EA0 = 0;		// Enable? again setup
				TZ0.TIORA.BIT.IOA = 2;	// Will set to 1 when timer happens
				DSERVOTOGGLE(42);		// BUGBUG:: For debug servo stuff.
				g_hservoD.abActivePins[0] = g_hservoD.iServo;	// remember which pin we are using
				input(g_hservoD.iServo);						// make sure the backing pin floats for now
			}
			
			g_hservoD.iServo = g_aabMuXServoMapTable[g_hservoD.bChannel][1];
			if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
				IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
				TZ0.GRB = (65535 - A32_TZF) - g_hservoD.ahsi[g_hservoD.iServo].wPW;  // 
				TZ0.TSR.BIT.IMFB = 0;		// Make sure the flag is clear
				TZ0.TIER.BIT.IMIEB = 1;		// enable that interrupt
				//TZ.TOCR.BIT.TOB0 = 0;		// Probably can just set this at init.
				TZ.TOER.BIT.EB0 = 0;		// Enable? again setup
				TZ0.TIORA.BIT.IOB = 2;	// Will set to 1 when timer happens
				DSERVOTOGGLE(42);		// BUGBUG:: For debug servo stuff.
				g_hservoD.abActivePins[1] = g_hservoD.iServo;	// remember which pin we are using
				input(g_hservoD.iServo);						// make sure the backing pin floats for now
			}

			g_hservoD.iServo = g_aabMuXServoMapTable[g_hservoD.bChannel][2];
			if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
				IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
				TZ0.GRC = (65535 - A32_TZF) - g_hservoD.ahsi[g_hservoD.iServo].wPW;  // 
				TZ0.TSR.BIT.IMFC = 0;		// Make sure the flag is clear
				TZ0.TIER.BIT.IMIEC = 1;		// enable that interrupt
				//TZ.TOCR.BIT.TOC0 = 0;		// Probably can just set this at init.
				TZ.TOER.BIT.EC0 = 0;		// Enable? again setup
				TZ0.TIORC.BIT.IOC = 2;	// Will set to 1 when timer happens
				DSERVOTOGGLE(42);		// BUGBUG:: For debug servo stuff.
				g_hservoD.abActivePins[2] = g_hservoD.iServo;	// remember which pin we are using
				input(g_hservoD.iServo);						// make sure the backing pin floats for now
			}

			g_hservoD.iServo = g_aabMuXServoMapTable[g_hservoD.bChannel][3];
			if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
				IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
				TZ0.GRD = (65535 - A32_TZF) - g_hservoD.ahsi[g_hservoD.iServo].wPW;  // 
				TZ0.TSR.BIT.IMFD = 0;		// Make sure the flag is clear
				TZ0.TIER.BIT.IMIED = 1;		// enable that interrupt
				//TZ.TOCR.BIT.TOD0 = 0;		// Probably can just set this at init.
				TZ.TOER.BIT.ED0 = 0;		// Enable? again setup
				TZ0.TIORC.BIT.IOD = 2;	// Will set to 1 when timer happens
				DSERVOTOGGLE(42);		// BUGBUG:: For debug servo stuff.
				g_hservoD.abActivePins[3] = g_hservoD.iServo;	// remember which pin we are using
				input(g_hservoD.iServo);						// make sure the backing pin floats for now
			}

			// Replicate for the other 4 channels on TimerZ1... 
			g_hservoD.iServo = g_aabMuXServoMapTable[g_hservoD.bChannel][4];
			if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
				IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
				TZ1.GRA = (65535 - A32_TZF) - g_hservoD.ahsi[g_hservoD.iServo].wPW;  // 
				TZ1.TSR.BIT.IMFA = 0;		// Make sure the flag is clear
				TZ1.TIER.BIT.IMIEA = 1;		// enable that interrupt
				//TZ.TOCR.BIT.TOA1 = 0;		// Probably can just set this at init.
				TZ.TOER.BIT.EA1 = 0;		// Enable? again setup
				TZ1.TIORA.BIT.IOA = 2;	// Will set to 1 when timer happens
				DSERVOTOGGLE(42);		// BUGBUG:: For debug servo stuff.
				g_hservoD.abActivePins[4] = g_hservoD.iServo;	// remember which pin we are using
				input(g_hservoD.iServo);						// make sure the backing pin floats for now
			}
			
			
			g_hservoD.iServo = g_aabMuXServoMapTable[g_hservoD.bChannel][5];
			if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
				IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
				TZ1.GRB = (65535 - A32_TZF) - g_hservoD.ahsi[g_hservoD.iServo].wPW;  // 
				TZ1.TSR.BIT.IMFB = 0;		// Make sure the flag is clear
				TZ1.TIER.BIT.IMIEB = 1;		// enable that interrupt
				//TZ.TOCR.BIT.TOB1 = 0;		// Probably can just set this at init.
				TZ.TOER.BIT.EB1 = 0;		// Enable? again setup
				TZ1.TIORA.BIT.IOB = 2;	// Will set to 1 when timer happens
				DSERVOTOGGLE(42);		// BUGBUG:: For debug servo stuff.
				g_hservoD.abActivePins[5] = g_hservoD.iServo;	// remember which pin we are using
				input(g_hservoD.iServo);						// make sure the backing pin floats for now
			}

			g_hservoD.iServo = g_aabMuXServoMapTable[g_hservoD.bChannel][6];
			if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
				IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
				TZ1.GRC = (65535 - A32_TZF) - g_hservoD.ahsi[g_hservoD.iServo].wPW;  // 
				TZ1.TSR.BIT.IMFC = 0;		// Make sure the flag is clear
				TZ1.TIER.BIT.IMIEC = 1;		// enable that interrupt
				//TZ.TOCR.BIT.TOC1 = 0;		// Probably can just set this at init.
				TZ.TOER.BIT.EC1 = 0;		// Enable? again setup
				TZ1.TIORC.BIT.IOC = 2;	// Will set to 1 when timer happens
				DSERVOTOGGLE(42);		// BUGBUG:: For debug servo stuff.
				g_hservoD.abActivePins[6] = g_hservoD.iServo;	// remember which pin we are using
				input(g_hservoD.iServo);						// make sure the backing pin floats for now
			}

			g_hservoD.iServo = g_aabMuXServoMapTable[g_hservoD.bChannel][7];
			if (g_hservoD.ahsi[g_hservoD.iServo].wPW) {
				IncrementalUpdateServoPos(&g_hservoD.ahsi[g_hservoD.iServo]);
				TZ1.GRD = (65535 - A32_TZF) - g_hservoD.ahsi[g_hservoD.iServo].wPW;  // 
				TZ1.TSR.BIT.IMFD = 0;		// Make sure the flag is clear
				TZ1.TIER.BIT.IMIED = 1;		// enable that interrupt
				//TZ.TOCR.BIT.TOD1 = 0;		// Probably can just set this at init.
				TZ.TOER.BIT.ED1 = 0;		// Enable? again setup
				TZ1.TIORC.BIT.IOD = 2;	// Will set to 1 when timer happens
				DSERVOTOGGLE(42);		// BUGBUG:: For debug servo stuff.
				g_hservoD.abActivePins[7] = g_hservoD.iServo;	// remember which pin we are using
				input(g_hservoD.iServo);						// make sure the backing pin floats for now
			}
			
			// End of timerz1 setup...			

		} else {
			// Later: Add analog pin reading 	
		}
	}
}

//=============================================================================
// TIMERZ1-Arc32() - For now the only timerZ1 function.  Only needs to handle
// the GRA-GRD interrupt asn timerz0 will do all of the setup for overflow...
//
//=============================================================================
extern "C" void TIMERZ1() __attribute__ ((interrupt_handler));
extern "C" void TIMERZ1()
{
	DSERVOTOGGLE(40);		// BUGBUG:: For debug servo stuff.

	if (TZ1.TSR.BIT.IMFA && TZ1.TIER.BIT.IMIEA) {
		DSERVOTOGGLE(41);		// BUGBUG:: For debug servo stuff.

		TZ1.GRA = 0xffff;			// should trigger and do its processing before overflow happens.
		TZ1.TIORA.BIT.IOA = 0x001;	// Will set to zero when timer happens
		TZ1.TIER.BIT.IMIEA = 0;		// don't need interrupt here
		TZ1.TSR.BIT.IMFA = 0;
	}	
	if (TZ1.TSR.BIT.IMFB && TZ1.TIER.BIT.IMIEB) {
		DSERVOTOGGLE(41);		// BUGBUG:: For debug servo stuff.
		TZ1.GRB = 0xffff;			// should trigger and do its processing before overflow happens.
		TZ1.TIORA.BIT.IOB = 0x001;	// Will set to zero when timer happens
		TZ1.TIER.BIT.IMIEB = 0;		// don't need interrupt here
		TZ1.TSR.BIT.IMFB = 0;
	}	
	if (TZ1.TSR.BIT.IMFC && TZ1.TIER.BIT.IMIEC) {
		DSERVOTOGGLE(41);		// BUGBUG:: For debug servo stuff.
		TZ1.GRC = 0xffff;			// should trigger and do its processing before overflow happens.
		TZ1.TIORC.BIT.IOC = 0x001;	// Will set to zero when timer happens
		TZ1.TIER.BIT.IMIEC = 0;		// don't need interrupt here
		TZ1.TSR.BIT.IMFC = 0;
	}	

	if (TZ1.TSR.BIT.IMFD && TZ1.TIER.BIT.IMIED) {
		DSERVOTOGGLE(41);		// BUGBUG:: For debug servo stuff.
		TZ1.GRD = 0xffff;			// should trigger and do its processing before overflow happens.
		TZ1.TIORC.BIT.IOD = 0x001;	// Will set to zero when timer happens
		TZ1.TIER.BIT.IMIED = 0;		// don't need interrupt here
		TZ1.TSR.BIT.IMFD = 0;
	}	
}



#endif


//==============================================================================
// Init the ServoGroupMove data.
//==============================================================================
void HServoInit(void)			// init our stuff
{
#ifdef DEBUG_HSERVO_COMMANDS
	DBGPrintf("HServoInit\r");
#endif
#ifdef BASICATOMPRO28
	TW.TCNT = 0;				// give us some delay before the first interrupt...
	TW.TMRW.BYTE = 0x80;		// CTS clock no PWMD...
	TW.TSRW.BYTE = 0;			// clear out all status bytes
	TW.TIOR0.BIT.IOA = 0;		// output match - don't change any io pins
	TW.TIOR0.BIT.IOB = 0;		// output match - don't change any io pins
	TW.TIOR1.BIT.IOC = 0;		// output match - don't change any io pins
	TW.TIOR1.BIT.IOD = 0;		// output match - don't change any io pins
	
	g_hservoD.abActivePins[0] = 0xff;
	g_hservoD.abActivePins[1] = 0xff;
	g_hservoD.abActivePins[2] = 0xff;
	g_hservoD.abActivePins[3] = 0xff;
	g_hservoD.iServo = 0xff; 		// start off with a high value to start off at 0

	TW.TIERW.BIT.OVIE = 1;		// Interrupt on overflow

#else //BAP40 and arc32

	TZ0.TCR.BYTE = 0;			// set clocks and the like.
	TZ1.TCR.BYTE = 0;			// set clocks and the like.
	TZ0.TCNT = 0;				// give us some delay before the first interrupt...
	TZ1.TCNT = 0;				// give us some delay before the first interrupt...
	TZ.TSTR.BIT.STR1 = 1;		// Make sure Timer Z1 is running 
	TZ.TSTR.BIT.STR0 = 1;		// Make sure Timer Z0 is running 

	TZ0.TSR.BYTE = 0;			// clear out all status bytes
		TZ1.TSR.BYTE = 0;			// clear out all status bytes
	TZ0.TIORA.BIT.IOA = 0;		// output match - don't change any io pins
	TZ0.TIORA.BIT.IOB = 0;		// output match - don't change any io pins
	TZ0.TIORC.BIT.IOC = 0;		// output match - don't change any io pins
	TZ0.TIORC.BIT.IOD = 0;		// output match - don't change any io pins
	
	int i;
	for (i=0; i< NUMTIMERCOMPAREMATCH; i++) {			// BUGBUG:: Unwind these functions into actual quick code...
		g_hservoD.abActivePins[i] = 0xff;
	}		

	for (i=0; i < NUMSERVOS; i++)
		g_hservoD.ahsi[i].wPW = 0;		// see if we need to init these...
		

	// if we are running on Arc32 - switch which function is handling the timerz functions...
	if (g_bWhichBap == H8_3687_ARC32) { 
		TZ1.TIORA.BIT.IOA = 0;		// output match - don't change any io pins
		TZ1.TIORA.BIT.IOB = 0;		// output match - don't change any io pins
		TZ1.TIORC.BIT.IOC = 0;		// output match - don't change any io pins
		TZ1.TIORC.BIT.IOD = 0;		// output match - don't change any io pins
		TZ.TMDR.BIT.SYNC = 1;		// lets sync the two timer counters...

		g_hservoD.bChannel = 0xff;		// set it up to overflow to 0 for the first one
		g_hservoD.ulTimerOverflow = 0;	// Count of timer overflows.
		output(46);					// Setup the two channels to output
		output(47);
		for (int i=48; i< 56; i++)
			input(i);				// leave the 8 outputs set to timer as input so they will
									// float when we are not having the timer drive them

#ifdef DEBUG_SERVO_PULSE		
		output(40);		// BUGBUG:: For debug servo stuff.
		output(41);		// BUGBUG:: For debug servo stuff.
		output(42);		// BUGBUG:: For debug servo stuff.
		output(43);		// BUGBUG:: For debug servo stuff.
#endif		
	}

	TZ0.TIER.BIT.OVIE = 1;		// Interrupt on overflow

#endif
			
	enable();					// Make sure interrupts are enabled...

}

//==============================================================================
// HServo - start up a group move...
//==============================================================================
void HServoGroupMoveStart(void)						// start a group move
{
	u8 i;

#ifdef DEBUG_HSERVO_COMMANDS
	DBGPrintf("HServoGroupMoveStart\r");
#endif	
	// Start up a group move.  If we are not already in a group move
	// initialize the group move code so we can detect which servos
	// we will process...
	if (g_hservoD.bGroupMove++ == 0) {
		for (i=0; i < NUMSERVOS; i++)
			g_hservoD.wGroupMovePW[i] = 0xffff;		// set to value we can detect
	}			
}

//==============================================================================
// HServoGroupMoveCommit - Commit any group move up a group that is in process
//==============================================================================
void HServoGroupMoveCommit(u16 wMoveTime)			// Do the commit, pass in how long you wish for the move to take
{
	u8 		i;
#ifdef DEBUG_HSERVO_COMMANDS
	DBGPrintf("HServoGroupMoveCommit %d\r", wMoveTime);
#endif
	if (wMoveTime)
		wMoveTime = (wMoveTime + 10)/20;			// convert to number of servo cycles.
	// Now do the actual group move... Detect count out of whack...
	if (g_hservoD.bGroupMove) {
		if (--g_hservoD.bGroupMove == 0) {
			// Ok it is time to commit all of the values...
			for (i=0; i < NUMSERVOS; i++) {
				if (g_hservoD.wGroupMovePW[i] != 0xffff) {
					// need to keep track of how many servos we have active 
					if (g_hservoD.wGroupMovePW[i] && !g_hservoD.ahsi[i].wPW) {
						if (++g_hservoD.cServosActive == 1)
							HServoInit();
					} else if (!g_hservoD.wGroupMovePW[i] && g_hservoD.ahsi[i].wPW) {
						g_hservoD.cServosActive--;
					}
					g_hservoD.ahsi[i].wDeltaPW = 0;								// Assume no delta, set last to minimize race conditions.
					if ((wMoveTime == 0) || (g_hservoD.ahsi[i].wPW == 0)) {
						g_hservoD.ahsi[i].wPW = g_hservoD.wGroupMovePW[i];		// set the PW to the new value
						g_hservoD.ahsi[i].wPWEnd =  g_hservoD.wGroupMovePW[i];	// Set the desired to the same
					} else {
						// First if we have not already made it to a previous timed
						g_hservoD.ahsi[i].wPW =  g_hservoD.ahsi[i].wPWEnd; 		// move location, get there now...
						g_hservoD.ahsi[i].wPWEnd =  g_hservoD.wGroupMovePW[i];	// Set the desired to the new time
						
						// calculate the delta per cycle..
						g_hservoD.ahsi[i].wDeltaPW = (iabs(g_hservoD.ahsi[i].wPWEnd - 
								g_hservoD.ahsi[i].wPW) + wMoveTime/2) / wMoveTime;
					}					
				}
			}
		}
	}	
#if 0	
	for (i=0; i < NUMSERVOS; i++) {
		DBGPrintf("%d - %x %x %x\r", i, g_hservoD.ahsi[i].wPW, g_hservoD.ahsi[i].wPWEnd, g_hservoD.ahsi[i].wDeltaPW);
	}
#endif	
}


//==============================================================================
// HServo - start up a group move...
//==============================================================================
void HServo(u8 pin, u16 wPulseWidth)
{
#ifdef DEBUG_HSERVO_COMMANDS
	DBGPrintf("HServo %d %d\r", pin, wPulseWidth);
#endif
	// First convert the pulse width to clocks for the processor...  
	wPulseWidth *= clockCyclesPerMicrosecond;			// Clocks per millisecond.
	
	output(pin);	
	// See if we are in a group move...
	if (g_hservoD.bGroupMove) {
		g_hservoD.wGroupMovePW[pin] = wPulseWidth;
	} else {
		// need to keep track of how many servos we have active 
		if (wPulseWidth && !g_hservoD.ahsi[pin].wPW) {
			if (++g_hservoD.cServosActive == 1)
				HServoInit();
		} else if (!wPulseWidth && g_hservoD.ahsi[pin].wPW) {
			g_hservoD.cServosActive--;
		}
		
		g_hservoD.ahsi[pin].wDeltaPW = 0;			// No delta time stuff
		g_hservoD.ahsi[pin].wPW = wPulseWidth;		// set the PW to the new value
		g_hservoD.ahsi[pin].wPWEnd =  wPulseWidth;	// Set the desired to the same
	}
}


//==============================================================================
// Init the HServoIdle data.
//==============================================================================
bool HServoIdle(u8 pin)
{
	return (g_hservoD.ahsi[pin].wPW == g_hservoD.ahsi[pin].wPWEnd);
}


//==============================================================================
// Init the HServoPos data.
//==============================================================================
extern u16 HServoPos(u8 pin)
{
	return g_hservoD.ahsi[pin].wPW / clockCyclesPerMicrosecond;
}



#ifdef H8_FAMILY_3687
//==============================================================================
// See if we can update Interrupt vector
//==============================================================================

PINTHANDLER UpdateHandler(int iVect, PINTHANDLER pinth)
{
	u16	*pwVect;
	u16	wOld;
	
	pwVect = (u16*)(iVect*2);		// Get to the address of the vector item.
	wOld = *pwVect;					// get theold value
	*pwVect = (u16)(void*)pinth;
	
	DBGPrintf("UV %d from %s to %x\n\r", iVect, wOld, (u16)(void*)pinth);
	
	return (PINTHANDLER)wOld;	
}
#endif
